# The LearningOnline Network
# Announce
#
# $Id: lonannounce.pm,v 1.7 2002/09/11 15:01:43 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonannounce;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;

sub readcalendar {
    my $courseid=shift;
    my $coursenum=$ENV{'course.'.$courseid.'.num'};
    my $coursedom=$ENV{'course.'.$courseid.'.domain'};
    my %thiscal=&Apache::lonnet::dump('calendar',$coursedom,$coursenum);
    my %returnhash=();
    foreach (keys %thiscal) {
        unless (($_=~/^error\:/) || ($thiscal{$_}=~/^error\:/)) {
	   $returnhash{$courseid.'@'.$_}=$thiscal{$_};
        }
    }
    return %returnhash;
}

sub emptycell {
    return '<td bgcolor="#AAAAAA">&nbsp;</td>';
}

sub normalcell {
    my ($day,$text)=@_;
    my $output='';
    foreach (split(/\_\_\_\&\&\&\_\_\_/,$text)) {
        if ($_) {
	    my ($courseid,$start,$end,@msg)=split(/\@/,$_);
            my $msg=join('@',@msg);
            my $fullmsg=$ENV{'course.'.$courseid.'.description'}.': '.$msg;
            if ($courseid eq $ENV{'request.course.id'}) {
              if (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) {
               $output.='<input type="checkbox" name="remove_'.$start.'_'.
		   $end.'">';
	      }
	    }
            $output.='<a href="javascript:alert('."'$fullmsg'".')">'.
	       substr($msg,0,20).'...</a><br>';
       }
    }
    return '<td><b>'.$day.'</b><br>'.$output.'</td>';
}

sub nextday {
    my %th=@_;
    $th{'day'}++;
    return (&Apache::loncommon::maketime(%th),$th{'month'});
}

sub showday {
    my ($tk,%allcal)=@_;
    my %th=&Apache::loncommon::timehash($tk);
    my ($nextday,$nextmonth)=&nextday(%th);
    my $outp='';
    my $oneday=24*3600;
    foreach (keys %allcal) {
	my ($course,$startdate,$enddate)=($_=~/^(\w+)\@(\d+)\_(\d+)$/);
        if (($startdate<$nextday) && ($enddate>$tk))  {
	    $outp.='___&&&___'.$course.'@'.$startdate.'@'.$enddate.'@'.
            $allcal{$_};
        }
    }
    return ($nextday,$nextmonth,&normalcell($th{'day'},$outp));
}

sub handler {
    my $r = shift;
    $r->content_type('text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# ---------------------------------------------------------- Get time right now
    my $today=time;
    my %todayhash=&Apache::loncommon::timehash($today);

# ---------------------------------------------------------- Get month and year
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
                                            ['month','year']);
# --------------------------------------------------- Decide what month to show
    my $year=$todayhash{'year'};
    if ($ENV{'form.year'}) { $year=$ENV{'form.year'}; }
    my $month=$todayhash{'month'};
    if ($ENV{'form.month'}) { $month=$ENV{'form.month'}; }
# --------------------------------------------- Find out first day of the month

    my %firstday=&Apache::loncommon::timehash(
       &Apache::loncommon::maketime( 'day' => 1, 'month'=> $month,
                                     'year' => $year, 'hours' => 0,
				     'minutes' => 0, 'seconds' => 0,
                                     'dlsav' => $todayhash{'dlsav'} ));
    my $weekday=$firstday{'weekday'};
# ------------------------------------------------------------ Print the screen
    $r->print(<<ENDDOCUMENT);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
<script>
    function pclose() {
        parmwin=window.open("/adm/rat/empty.html","LONCAPAparms",
                 "height=350,width=350,scrollbars=no,menubar=no");
        parmwin.close();
    }

    function pjump(type,dis,value,marker,ret,call) {
        parmwin=window.open("/adm/rat/parameter.html?type="+escape(type)
                 +"&value="+escape(value)+"&marker="+escape(marker)
                 +"&return="+escape(ret)
                 +"&call="+escape(call)+"&name="+escape(dis),"LONCAPAparms",
                 "height=350,width=350,scrollbars=no,menubar=no");

    }

    function dateset() {
        if (document.anno.pres_marker.value=='end') {
           document.anno.enddate.value=
	       document.anno.pres_value.value;
        }
        if (document.anno.pres_marker.value=='start') {
           document.anno.startdate.value=
	       document.anno.pres_value.value;
        }
        pclose();
    }

    function trysubmit() {
        if (document.anno.startdate.value=='') {
	    alert("Announcements must have a starting date");
            return '';
        }
        if (document.anno.enddate.value=='') {
	    alert("Announcements must have a starting date");
            return '';
        }
        if (document.anno.startdate.value>document.anno.enddate.value) {
	    alert("Starting date is later than ending date");
            return '';
        }
        document.anno.action.value="new";
	document.anno.submit();
    }

    function removesub() {
        document.anno.action.value="del";
	document.anno.submit();
    }
</script>
</head>
ENDDOCUMENT
    $r->print(&Apache::loncommon::bodytag("Announcements and Calendar"));
# does this user have privileges to post, etc?
    my $allowed=0;
    if ($ENV{'request.course.id'}) {
       $allowed=&Apache::lonnet::allowed('srm',$ENV{'request.course.id'});
    }

    if ($allowed) {
        my $coursenum=$ENV{'course.'.$ENV{'request.course.id'}.'.num'};
        my $coursedom=$ENV{'course.'.$ENV{'request.course.id'}.'.domain'};
# ----------------------------------------------------- Store new submitted one
        if ($ENV{'form.action'} eq 'new') {
	    &Apache::lonnet::put('calendar',{ 
		$ENV{'form.startdate'}.'_'.$ENV{'form.enddate'} => 
		    $ENV{'form.msg'} },$coursedom,$coursenum);
        }
# ---------------------------------------------------------------- Remove items
        if ($ENV{'form.action'} eq 'del') {
	    my @delwhich=();
            foreach (keys %ENV) {
		if ($_=~/^form\.remove\_(.+)$/) {
		    push(@delwhich,$1);
                }
            }
            &Apache::lonnet::del('calendar',\@delwhich,$coursedom,$coursenum);
        }
# -------------------------------------------------------- Form to post new one
        my %tomorrowhash=%todayhash;
        $tomorrowhash{'day'}++;
        my $tomorrow=&Apache::loncommon::maketime(%tomorrowhash);
        
	$r->print(<<ENDFORM);
<form name="anno" method="post">
<input type="hidden" value=''          name="action"      >
<input type="hidden" value=''          name="pres_value"  >
<input type="hidden" value=''          name="pres_type"   >
<input type="hidden" value=''          name="pres_marker" >
<input type="hidden" value='$today'    name="startdate"   >
<input type="hidden" value='$tomorrow' name="enddate"     >
<a 
 href="javascript:pjump('date_start','Starting Date',document.anno.startdate.value,'start','anno.pres','dateset');"
>Set Starting Date</a>
<a 
 href="javascript:pjump('date_end','Ending Date',document.anno.enddate.value,'end','anno.pres','dateset');"
>Set Ending Date</a><p>
<textarea name="msg" rows="4" cols="60">
</textarea>
<input type="button" onClick="trysubmit()" value="Post Announcement"><hr>
<input type="button" onClick="removesub()" value="Remove Marked"><hr>
ENDFORM
    }
# ----------------------------------------------------- Summarize all calendars
    my %allcal=();
    foreach (&Apache::loncommon::findallcourses()) {
	%allcal=(%allcal,&readcalendar($_));
    }

# ------------------------------- Initialize table and forward backward buttons
    my ($pm,$py,$fm,$fy)=($month-1,$year,$month+1,$year);
    if ($pm<1) { ($pm,$py)=(12,$year-1); }
    if ($fm>12){ ($fm,$fy)=(1,$year+1); }
    $r->print('<h1>'.('','January','February','March','April','May',
		      'June','July','August','September','October',
                      'November','December')[$month].' '.$year.'</h1>'.
 '<a href="/adm/announcements?month='.$pm.'&year='.$py.'">Previous Month</a> '.
 '<a href="/adm/announcements?month='.$fm.'&year='.$fy.'">Next Month</a><p>'.
        '<table border="2" cols="7" rows="5"><tr><th>Sun</th><th>Mon</th>'.
        '<th>Tue</th><th>Wed</th><th>Thu</th><th>Fri</th><th>Sat</th></tr>');

    my $tk=&Apache::loncommon::maketime(%firstday);
    my $outp;
    my $nm;

# ---------------------------------------------------------------- Actual table
    $r->print('<tr>');
    for (my $i=0;$i<$weekday;$i++) { $r->print(&emptycell); }
    for (my $i=$weekday;$i<=6;$i++) { 
        ($tk,$nm,$outp)=&showday($tk,%allcal);
        $r->print($outp);
    }
    $r->print('</tr>');

    for (my $k=0;$k<=3;$k++) {
        $r->print('<tr>');
        for (my $i=0;$i<=6;$i++) {
            ($tk,$nm,$outp)=&showday($tk,%allcal);
            if ($month!=$nm) { $outp=&emptycell; }
            $r->print($outp);
        }
        $r->print('</tr>');
    }
# ------------------------------------------------------------------- End table
    $r->print('</table>');
    if ($allowed) { $r->print('</form>'); }
    $r->print('<p>'.
 '<a href="/adm/announcements?month='.$pm.'&year='.$py.'">Previous Month</a> '.
 '<a href="/adm/announcements?month='.$fm.'&year='.$fy.'">Next Month</a><p>'.
 '</body></html>');
    return OK;
} 

1;
__END__
