# The LearningOnline Network with CAPA
# (Publication Handler
#
# $Id: lonstudentassessment.pm,v 1.20 2002/12/11 18:41:00 minaeibi Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (Navigate problems for statistical reports
# YEAR=2001
# 5/5,7/9,7/25/1,8/11,9/13,9/26,10/5,10/9,10/22,10/26 Behrouz Minaei
# 11/1,11/4,11/16,12/14,12/16,12/18,12/20,12/31 Behrouz Minaei
# YEAR=2002
# 1/22,2/1,2/6,2/25,3/2,3/6,3/17,3/21,3/22,3/26,4/7,5/6 Behrouz Minaei
# 5/12,5/14,5/15,5/19,5/26,7/16  Behrouz Minaei
#
###

package Apache::lonstudentassessment; 

use strict;
use Apache::lonhtmlcommon;
use Apache::loncoursedata;
use GDBM_File;

#my $jr;

sub BuildStudentAssessmentPage {
    my ($cacheDB,$students,$courseID,$formName,$headings,$spacing,
        $studentInformation,$r,$c)=@_;
#    $jr = $r;
    my %cache;
    unless(tie(%cache,'GDBM_File',$cacheDB,&GDBM_READER(),0640)) {
        $r->print('<html><body>Unable to tie database.</body></html>');
        return;
    }

    # Remove students who don't have the proper section.
    my @sectionsSelected = split(':',$cache{'sectionsSelected'});
    for(my $studentIndex=((scalar @$students)-1); $studentIndex>=0;
        $studentIndex--) {
        my $value = $cache{$students->[$studentIndex].':section'};
        my $found = 0;
        foreach (@sectionsSelected) {
            if($_ eq 'none') {
                if($value eq '' || !defined($value) || $value eq ' ') {
                    $found = 1;
                    last;
                }
            } else {
                if($value eq $_) {
                    $found = 1;
                    last;
                }
            }
        }
        if($found == 0) {
            splice(@$students, $studentIndex, 1);
        }
    }
    my ($infoHeadings, $infoKeys, $sequenceHeadings, $sequenceKeys,
        $doNotShow) = 
        &ShouldShowColumns(\%cache, $headings, $studentInformation);

    my $selectedName = &FindSelectedStudent(\%cache, 
                                            $cache{'StudentAssessmentStudent'},
                                            $students);
    $r->print(&CreateInterface(\%cache, $selectedName, $students, $formName,
                               $doNotShow));
    $r->rflush();

    my $Str = '';
    if($selectedName eq 'No Student Selected') {
	$Str .= '<h3><font color=blue>WARNING: ';
        $Str .= 'Please select a student</font></h3>';
        $r->print($Str);
        return;
    }

    $r->print(&CreateTableHeadings(\%cache, $spacing, $infoKeys, $infoHeadings,
                                   $sequenceKeys, $sequenceHeadings));
    untie(%cache);
    if($c->aborted()) {  return $Str; }

    my $selected=0;
    $r->print('<pre>'."\n");
    foreach (@$students) {
        if($c->aborted()) { return $Str; }
        next if ($_ ne $selectedName &&
                 $selectedName ne 'All Students');
        $selected = 1;

        my @who = ($_);
        next if(&Apache::loncoursedata::DownloadStudentCourseData(\@who, 'true',
                                                             $cacheDB, 'true',
                                                             'false', $courseID,
                                                             $r, $c) ne 'OK');
        next if($c->aborted());

        if(tie(%cache,'GDBM_File',$cacheDB,&GDBM_READER(),0640)) {
            my @before=();
            my @after=();
            my @updateColumn=();
            my $foundUpdate = 0;
            foreach(@$infoKeys) {
                if(/updateTime/) {
                    $foundUpdate=1;
                    push(@updateColumn, $_);
                    next;
                }
                if($foundUpdate) {
                    push(@after, $_);
                } else {
                    push(@before, $_);
                }
            }
            my $displayString = 'DISPLAYDATA'.$spacing;
            $r->print(&Apache::lonhtmlcommon::FormatStudentInformation(
                                                         \%cache, $_,
                                                         \@before,
                                                         $displayString,
                                                         'preformatted'));

            if($foundUpdate) {
                $displayString = '';
                $displayString .= '<a href="/adm/statistics?reportSelected=';
                $displayString .= &Apache::lonnet::escape('Student Assessment');
                $displayString .= '&download='.$_.'">';
                $displayString .= 'DISPLAYDATA</a>'.$spacing;
                $r->print(&Apache::lonhtmlcommon::FormatStudentInformation(
                                                                   \%cache, $_,
                                                                   \@updateColumn,
                                                                   $displayString,
                                                                   'preformatted'));
            }

            $displayString = 'DISPLAYDATA'.$spacing;
            $r->print(&Apache::lonhtmlcommon::FormatStudentInformation(
                                                         \%cache, $_,
                                                         \@after,
                                                         $displayString,
                                                         'preformatted'));
            $r->print(&StudentReport(\%cache, $_, $spacing, $sequenceKeys));
            $r->print("\n");
            $r->rflush();
            untie(%cache);
        }
    }


#    $r->print("AverageTtal");
#    $r->print(&StudentAverageTotal(\%cache, $spacing, $sequenceKeys));
#    $r->print("\n");
#    $r->rflush();
    untie(%cache);


    $r->print('</pre>'."\n");
    if($selected == 0) {
	$Str .= '<h3><font color=blue>WARNING: ';
        $Str .= 'Please select a student</font></h3>';
        $r->print($Str);
    }

    return;
}

#---- Student Assessment Web Page --------------------------------------------

sub CreateInterface {
    my($cache,$selectedName,$students,$formName,$doNotShow)=@_;

    my $Str = '';
    $Str .= &CreateLegend();
    $Str .= '<table><tr><td>'."\n";
    $Str .= '<input type="submit" name="PreviousStudent" ';
    $Str .= 'value="Previous Student" />'."\n";
    $Str .= '&nbsp&nbsp&nbsp'."\n";
    $Str .= &Apache::lonhtmlcommon::StudentOptions($cache, $students,
                                                   $selectedName, 
                                                   'StudentAssessment', 
                                                   $formName);
    $Str .= "\n".'&nbsp&nbsp&nbsp'."\n";
    $Str .= '<input type="submit" name="NextStudent" ';
    $Str .= 'value="Next Student" />'."\n";
    $Str .= '</td></tr></table>'."\n";
    $Str .= '<table cellspacing="5"><tr>'."\n";
    $Str .= '<td align="center"><b>Select Sections</b>'."\n";
    $Str .= '</td>'."\n";
    $Str .= '<td align="center"><b>Select column to view:</b></td>'."\n";
    $Str .= '<td></td></tr>'."\n";

    $Str .= '<tr><td align="center">'."\n";
    my @sections = split(':',$cache->{'sectionList'});
    my @selectedSections = split(':',$cache->{'sectionsSelected'});
    $Str .= &Apache::lonhtmlcommon::MultipleSectionSelect(\@sections,
                                                          \@selectedSections,
                                                          'Statistics');
    $Str .= '</td><td align="center">';
    $Str .= &CreateColumnSelectionBox($doNotShow);
    $Str .= '</td><td>'."\n";
    $Str .= '<input type="submit" name="DefaultColumns" ';
    $Str .= 'value="Default Column Display" />'."\n";
    $Str .= '</td><td>'."\n";
    $Str .= '<input type="submit" name="displaymode" ';
    if (! exists($ENV{'form.displaymode'}) ||
        lc($ENV{'form.displaymode'}) eq 'display with links') {
        $Str .= 'value="Display without links" />';
        # Set the current value, in case it is undefined
        $ENV{'form.displaymode'} = 'Display with links'; 
    } else {
        $Str .= 'value="Display with links" />';
    }
    $Str .= "\n";
    $Str .= '</td></tr></table>'."\n";

    return $Str;
}

sub CreateTableHeadings {
    my($cache,$spacing,$infoKeys,$infoHeadings,$sequenceKeys,
       $sequenceHeadings)=@_;

    my $Str = '';
    $Str .= '<table border="0" cellpadding="0" cellspacing="0">'."\n";

    $Str .= '<tr>'."\n";
    $Str .= &CreateColumnSelectors($infoHeadings, $sequenceHeadings,
                                   $sequenceKeys);
    $Str .= '<td></td></tr>'."\n";

    $Str .= '<tr>'."\n";
    my $displayString = '<td align="left"><pre><a href="/adm/statistics?';
    $displayString .= 'sort=LINKDATA">DISPLAYDATA</a>FORMATTING';
    $displayString .= $spacing.'</pre></td>'."\n";
    $Str .= &Apache::lonhtmlcommon::CreateHeadings($cache,
                                                   $infoKeys,
                                                   $infoHeadings,
                                                   $displayString,
                                                   'preformatted');

    $displayString  = '<td align="left"><pre>DISPLAYDATAFORMATTING'.$spacing;
    $displayString .= '</pre></td>'."\n";
    $Str .= &Apache::lonhtmlcommon::CreateHeadings($cache,
                                                   $sequenceKeys,
                                                   $sequenceHeadings,
                                                   $displayString,
                                                   'preformatted');

    $Str .= '<td><pre>Total Solved/Total Problems</pre></td>';
    $Str .= '</tr></table>'."\n";

    return $Str;
}

=pod

=item &FormatStudentData()

First, FormatStudentInformation is called and prefixes the course information.
This function produces a formatted string of the student\'s course information.
Each column of data represents all the problems for a given sequence.  For
valid grade data, a link is created for that problem to a submission record
for that problem.

=over 4

Input: $name, $studentInformation, $ChartDB

$name: The name and domain of the current student in name:domain format

$studentInformation: A pointer to an array holding the names used to 
remove data from the hash.  They represent 
the name of the data to be removed.

$ChartDB: The name of the cached data database which will be tied to that 
database.

Output: $Str

$Str: Formatted string that is an entire row of the chart.  It is a 
concatenation of student information and student course information.

=back

=cut

sub StudentReport {
    my ($cache,$name,$spacing,$showSequences)=@_;
    my ($username,$domain)=split(':',$name);

    my $Str = '';
    if(defined($cache->{$name.':error'})) {
        return $Str;
    }
    if($cache->{$name.':error'} =~ /course/) {
        $Str .= '<b><font color="blue">No course data for student </font>';
        $Str .= '<font color="red">'.$username.'.</font></b><br>';
        return $Str;
    }

    my $hasVersion = 'false';
    my $hasFinalData = 'false';
    foreach my $sequence (@$showSequences) {
        my $hasData = 'false';
        my $characterCount=0;
        foreach my $problemID (split(':', $cache->{$sequence.':problems'})) {
            my $problem = $cache->{$problemID.':problem'};
            # All grades (except for versionless parts) are displayed as links
            # to their submission record.  Loop through all the parts for the
            # current problem in the correct order and prepare the output links
            foreach(split(/\:/,$cache->{$sequence.':'.$problemID.
                                        ':parts'})) {
                if($cache->{$name.':'.$problemID.':NoVersion'} eq 'true' ||
                   $cache->{$name.':'.$problemID.':'.$_.':code'} eq ' ' ||
                   $cache->{$name.':'.$problemID.':'.$_.':code'} eq '') {
                    $Str .= ' ';
                    $characterCount++;
                    next;
                }
                $hasVersion = 'true';
                $hasData = 'true';
                if (lc($ENV{'form.displaymode'}) ne 'display without links') {
                    $Str .= '<a href="/adm/grades?symb=';
                    $Str .= &Apache::lonnet::escape($problem);
                    $Str .= '&student='.$username.'&domain='.$domain;
                    $Str .= '&command=submission">'; 
                }
                my $code = $cache->{$name.':'.$problemID.':'.$_.':code'};
                my $tries = $cache->{$name.':'.$problemID.':'.$_.':tries'};
                if($code eq '*' && $tries < 10 && $tries ne '') {
                    $code = $tries;
                }
                $Str .= $code;
                if (lc($ENV{'form.displaymode'}) ne 'display without links') {
                    $Str .= '</a>';
                }
                $characterCount++;
            }
        }

        # Output the number of correct answers for the current sequence.
        # This part takes up 6 character slots, but is formated right
        # justified.
        my $spacesNeeded=$cache->{$sequence.':columnWidth'}-$characterCount;
        $spacesNeeded -= 3;
        $Str .= (' 'x$spacesNeeded);

#        my $outputProblemsCorrect = sprintf("%3d", $cache->{$name.':'.$sequence.
#							    ':problemsCorrect'});

	my $outputProblemsCorrect = sprintf("%2d/%2d", $cache->{$name.':'.$sequence.
                                            ':problemsCorrect'},
                                            $characterCount);
        if($hasData eq 'true') {
            $Str .= '<font color="#007700">'.$outputProblemsCorrect.'</font>';
            $hasFinalData = 'true';
        } else {
            $Str .= '<font color="#007700">     </font>';
        }
        $Str .= $spacing;
    }

    # Output the total correct problems over the total number of problems.
    # I don't like this type of formatting, but it is a solution.  Need
    # a way to dynamically determine the space requirements.
    my $outputProblemsSolved = sprintf("%4d", $cache->{$name.':problemsSolved'});
    my $outputTotalProblems  = sprintf("%4d", $cache->{$name.':totalProblems'});
    if($hasFinalData eq 'true') {
        $Str .= '<font color="#000088">'.$outputProblemsSolved.
	    ' / '.$outputTotalProblems.'</font>';
    } else {
        $Str .= '<font color="#000088">           </font>';
    }

    if($hasVersion eq 'false') {
        $Str = '<b><font color="blue">No course data.</font></b>';
    }

    return $Str;
}


sub StudentAverageTotal {
    
    return "";

    my ($cache,$name,$spacing,$showSequences)=@_;
    my $username = $name;
    my $Str = '';
    my $hasVersion = 'false';
    my $hasFinalData = 'false';
    foreach my $sequence (@$showSequences) {
        my $hasData = 'false';
        my $characterCount=0;
        foreach my $problemID (split(':', $cache->{$sequence.':problems'})) {
            my $problem = $cache->{$problemID.':problem'};
            # All grades (except for versionless parts) are displayed as links
            # to their submission record.  Loop through all the parts for the
            # current problem in the correct order and prepare the output links
            foreach(split(/\:/,$cache->{$sequence.':'.$problemID.
                                        ':parts'})) {
                if($cache->{$name.':'.$problemID.':NoVersion'} eq 'true' ||
                   $cache->{$name.':'.$problemID.':'.$_.':code'} eq ' ' ||
                   $cache->{$name.':'.$problemID.':'.$_.':code'} eq '') {
                    $Str .= ' ';
                    $characterCount++;
                    next;
                }
                $hasVersion = 'true';
                $hasData = 'true';
                $Str .= '<a href="/adm/grades?symb=';
                $Str .= &Apache::lonnet::escape($problem);
                $Str .= '&student= Average &domain= Total';
                $Str .= '&command=submission">';
                my $code = $cache->{$name.':'.$problemID.':'.$_.':code'};
                my $tries = $cache->{$name.':'.$problemID.':'.$_.':tries'};
                if($code eq '*' && $tries < 10 && $tries ne '') {
                    $code = $tries;
                }
                $Str .= $code;
                $Str .= '</a>';
                $characterCount++;
            }
        }

        # Output the number of correct answers for the current sequence.
        # This part takes up 6 character slots, but is formated right
        # justified.
        my $spacesNeeded=$cache->{$sequence.':columnWidth'}-$characterCount;
        $spacesNeeded -= 3;
        $Str .= (' 'x$spacesNeeded);

#        my $outputProblemsCorrect = sprintf("%3d", $cache->{$name.':'.$sequence.
#							    ':problemsCorrect'});

	my $outputProblemsCorrect = sprintf("%2d/%2d", $cache->{$name.':'.$sequence.
                                            ':problemsCorrect'},
                                            $characterCount);
        if($hasData eq 'true') {
            $Str .= '<font color="#007700">'.$outputProblemsCorrect.'</font>';
            $hasFinalData = 'true';
        } else {
            $Str .= '<font color="#007700">     </font>';
        }
        $Str .= $spacing;
    }

    # Output the total correct problems over the total number of problems.
    # I don't like this type of formatting, but it is a solution.  Need
    # a way to dynamically determine the space requirements.
    my $outputProblemsSolved = sprintf("%4d", $cache->{$name.':problemsSolved'});
    my $outputTotalProblems  = sprintf("%4d", $cache->{$name.':totalProblems'});
    if($hasFinalData eq 'true') {
        $Str .= '<font color="#000088">'.$outputProblemsSolved.
	    ' / '.$outputTotalProblems.'</font>';
    } else {
        $Str .= '<font color="#000088">           </font>';
    }

    if($hasVersion eq 'false') {
        $Str = '<b><font color="blue">No course data.</font></b>';
    }

    return $Str;
}



=pod

=item &CreateLegend()

This function returns a formatted string containing the legend for the
chart.  The legend describes the symbols used to represent grades for
problems.

=cut

sub CreateLegend {
    my $Str = "<p><pre>".
              "   1  correct by student in 1 try\n".
              "   7  correct by student in 7 tries\n".
              "   *  correct by student in more than 9 tries\n".
	      "   +  correct by hand grading or override\n".
              "   -  incorrect by override\n".
	      "   .  incorrect attempted\n".
	      "   #  ungraded attempted\n".
              "      not attempted (blank field)\n".
	      "   x  excused".
              "</pre><p>";
    return $Str;
}

=pod

=item &CreateColumnSelectionBox()

If there are columns not being displayed then this selection box is created
with a list of those columns.  When selections are made and the page
refreshed, the columns will be removed from this box and the column is
put back in the chart.  If there is no columns to select, no row is added
to the interface table.

=over 4
Input: $CacheData, $headings


$CacheData: A pointer to a hash tied to the cached data

$headings:  An array of the names of the columns for the student information.
They are used for displaying which columns are missing.

Output: $notThere

$notThere: The string contains one row of a table.  The first column has the
name of the selection box.  The second contains the selection box
which has a size of four.

=back

=cut

sub CreateColumnSelectionBox {
    my ($doNotShow)=@_;

    my $notThere = '';
    $notThere .= '<select name="ReselectColumns" size="4" ';
    $notThere .= 'multiple="true">'."\n";

    for(my $index=0; $index<$doNotShow->{'count'}; $index++) {
        my $name = $doNotShow->{$index.':name'};
        $notThere .= '<option value="';
        $notThere .= $doNotShow->{$index.':id'}.'">';
        $notThere .= $name.'</option>'."\n";
    }

    $notThere .= '</select>';

    return $notThere;
}

=pod

=item &CreateColumnSelectors()

This function generates the checkboxes above the column headings.  The
column will be removed if the checkbox is unchecked.

=over 4

Input: $CacheData, $headings

$CacheData: A pointer to a hash tied to the cached data

$headings:  An array of the names of the columns for the student 
information.  They are used to know what are the student information columns

Output: $present

$present: The string contains the first row of a table.  Each column contains
a checkbox which is left justified.  Currently left justification is used
for consistency of location over the column in which it presides.

=back

=cut

sub CreateColumnSelectors {
    my ($infoHeadings, $sequenceHeadings, $sequenceKeys)=@_;

    my $present = '';
    for(my $index=0; $index<(scalar @$infoHeadings); $index++) {
        $present .= '<td align="left">';
        $present .= '<input type="checkbox" checked="on" ';
        $present .= 'name="HeadingColumn'.$infoHeadings->[$index].'" />';
        $present .= '</td>'."\n";
    }

    for(my $index=0; $index<(scalar @$sequenceHeadings); $index++) {
        $present .= '<td align="left">';
        $present .= '<input type="checkbox" checked="on" ';
        $present .= 'name="SequenceColumn'.$sequenceKeys->[$index].'" />';
        $present .= '</td>'."\n";
    }

    return $present;
}

#---- END Student Assessment Web Page ----------------------------------------

#---- Student Assessment Worker Functions ------------------------------------

sub FindSelectedStudent {
    my($cache, $selectedName, $students)=@_;

    if($selectedName eq 'All Students' ||
       $selectedName eq 'No Student Selected') {
        return $selectedName;
    }

    for(my $index=0; $index<(scalar @$students); $index++) {
        my $fullname = $cache->{$students->[$index].':fullname'};
        if($fullname eq $selectedName) {
            if($cache->{'StudentAssessmentMove'} eq 'next') {
                if($index == ((scalar @$students) - 1)) {
                    $selectedName = $students->[0];
                    return $selectedName;
                } else {
                    $selectedName = $students->[$index+1];
                    return $selectedName;
                }
            } elsif($cache->{'StudentAssessmentMove'} eq 'previous') {
                if($index == 0) {
                    $selectedName = $students->[-1];
                    return $selectedName;
                } else {
                    $selectedName = $students->[$index-1];
                    return $selectedName;
                }
            } else {
                $selectedName = $students->[$index];
                return $selectedName;
            }
            last;
        }
    }

    return 'No Student Selected';
}

=pod

=item &ShouldShowColumn()

Determine if a specified column should be shown on the chart.

=over 4

Input: $cache, $test

$cache: A pointer to the hash tied to the cached data

$test: The form name of the column (heading.$headingIndex) or 
(sequence.$sequenceIndex)

Output: 0 (false), 1 (true)

=back

=cut

sub ShouldShowColumns {
    my ($cache,$headings,$cacheKey)=@_;

    my @infoKeys=();
    my @infoHeadings=();

    my @sequenceKeys=();
    my @sequenceHeadings=();

    my %doNotShow;

    my $index;
    my $count = 0;
    my $check = '';
    for($index=0; $index < scalar @$headings; $index++) {
        $check = 'HeadingColumn'.$headings->[$index];
        if($cache->{'HeadingsFound'} =~ /$check/) {
            push(@infoHeadings, $headings->[$index]);
            push(@infoKeys, $cacheKey->[$index]);
        } else {
            $doNotShow{$count.':name'} = $headings->[$index];
            $doNotShow{$count.':id'} = 'HeadingColumn'.$headings->[$index];
            $count++;
        }
    }

    foreach my $sequence (split(/\:/,$cache->{'orderedSequences'})) {
        $check = 'SequenceColumn'.$sequence;
        if($cache->{'SequencesFound'} eq 'All Sequences' || 
           $cache->{'SequencesFound'} =~ /$check/) {
            push(@sequenceHeadings, $cache->{$sequence.':title'});
            push(@sequenceKeys, $sequence);
        } else {
            $doNotShow{$count.':name'} = $cache->{$sequence.':title'};
            $doNotShow{$count.':id'} = 'SequenceColumn'.$sequence;
            $count++;
        }
    }

    $doNotShow{'count'} = $count;

    return (\@infoHeadings, \@infoKeys, \@sequenceHeadings, 
            \@sequenceKeys, \%doNotShow);
}

#---- END Student Assessment Worker Functions --------------------------------

1;
__END__
